package it.danieleverducci.subitobeers.ui

import android.app.DatePickerDialog
import android.content.res.Resources
import android.os.Bundle
import android.view.*
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import android.widget.Toast
import it.danieleverducci.subitobeers.BeerNavigation
import it.danieleverducci.subitobeers.BeersRepository
import it.danieleverducci.subitobeers.R
import it.danieleverducci.subitobeers.databinding.FragmentBeersListBinding
import it.danieleverducci.subitobeers.entities.Beer
import it.danieleverducci.subitobeers.networking.BeersFilter
import java.util.*

/**
 * A fragment representing a list of Items.
 */
class BeersFragment : Fragment(), BeersRepository.Listener, BeerRecyclerAdapter.Listener {

    enum class FILTERTYPE {SINCE, TO}

    private val rvAdapter = BeerRecyclerAdapter()
    private val repo = BeersRepository()
    lateinit var binding: FragmentBeersListBinding

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        setHasOptionsMenu(true);

        // Inflate layout
        binding = FragmentBeersListBinding.inflate(
            LayoutInflater.from(container!!.context),
            container,
            false
        )

        // Register filter buttons listeners
        binding.listFilterSinceBt.setOnClickListener { onFilterButtonClicked(FILTERTYPE.SINCE) }
        binding.listFilterToBt.setOnClickListener { onFilterButtonClicked(FILTERTYPE.TO) }

        // Set the adapter
        with(binding.list) {
            layoutManager = LinearLayoutManager(context)
            adapter = rvAdapter
        }

        // Register for recyclerview adapter events
        rvAdapter.listener = this

        // Register for network completed events
        repo.listener = this

        // Load first page
        loadBeers(true)

        return binding.root
    }

    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        super.onCreateOptionsMenu(menu, inflater)
        requireActivity().menuInflater.inflate(R.menu.list_menu, menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == R.id.action_filter)
            toggleFilter()
        return super.onOptionsItemSelected(item)
    }

    override fun onBeersObtained(beers: List<Beer>, more: Boolean) {
        rvAdapter.addItems(beers, more)
    }

    override fun onFailure() {
        Toast.makeText(context, R.string.network_error, Toast.LENGTH_SHORT).show()
    }

    /**
     * Called when an item is clicked in the list
     */
    override fun OnItemClicked(item: Beer) {
        if (activity is BeerNavigation)
            (activity as BeerNavigation).showBeerDetail(item);
        else
            throw IllegalStateException("Activity must implement BeerNavigation")
    }

    /**
     * Called when a filter button is clicked
     * @param type whether is start or end date
     */
    fun onFilterButtonClicked(type: FILTERTYPE) {
        val dpd = DatePickerDialog(requireContext())
        // Hide day selector
//        dpd.datePicker.findViewById<View>(
//            Resources.getSystem().getIdentifier("day", "id", "android")
//            ).visibility = View.GONE
        dpd.setOnDateSetListener(DatePickerDialog.OnDateSetListener { view, year, month, dayOfMonth ->
            // On date chosen
            val formattedDate = "${String.format("%02d", month)}-${String.format("%04d", year)}"
            if (type == FILTERTYPE.SINCE)
                repo.filter.brewedAfter = formattedDate
            else
                repo.filter.brewedBefore = formattedDate
            // Update list
            loadBeers(true)
        })
        dpd.show()
    }

    /**
     * Called when last list item is displayed.
     * Used to fetch more elements
     */
    override fun OnLastItemScrolled() {
        repo.page++
        loadBeers(false)
    }

    /**
     * Toggles beer filter visibility
     */
    private fun toggleFilter() {
        binding.listFilterDropdown.visibility =
            if (binding.listFilterDropdown.visibility == View.VISIBLE) View.GONE else View.VISIBLE
    }

    /**
     * Loads beers from network from first page
     * @param clear: clears the list and fetch from first element
     */
    private fun loadBeers(clear: Boolean) {
        if (clear) {
            rvAdapter.clear()
            repo.reset()
        }

        // Load with filter
        repo.getBeers()
    }

}